/*
 * MoGraSo: A programme for the calculation of the response of fibre grating
 * elements using modal methods.
 *
 * MoGraSo Copyright (C) 2007 Hannes Hochreiner <hannes@hochreiner.net>
 *
 * This file is part of MoGraSo.
 *
 * MoGraSo is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * MoGraSo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * MoGraSo.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include <iostream>
#include <fstream>
#include <string>
#include <stdexcept>
#include <vector>

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include "mograsoJobFactory.h"

int main (int argc, char * const argv[]) {
	std::string input_file_name;

#ifdef HAVE_CONFIG_H
	std::cout << PACKAGE_STRING << "\n\n";
	std::cout << "Copyright 2007, 2008 Hannes Hochreiner\n";
	std::cout << "MoGraSo is distributed under the GNU General Public License version 3 or later as published by the Free Software Foundation. For details on the license see <http://www.gnu.org/licenses/>.\n\n";
	std::cout << "For information about this programme or bug reports please contact: " << PACKAGE_BUGREPORT << "\n";
#endif

	if ((argc == 3) && (std::string(argv[1]) == "-i"))
		input_file_name = std::string(argv[2]);
	else
	{
		std::cout << "Please specify the input file: ";
		std::cin >> input_file_name;
	}
		
	mograsoJobFactory jobFactory;
	jobFactory.process_file(stows(input_file_name));

	return 0;
}

template<class T>
std::vector<T> randomiseVector(const std::vector<T>& vector, unsigned int seed)
{
	std::vector<T> output_vector(vector);
	srand(seed);
	
	for (int loopCounter = 0; loopCounter < 2 * output_vector.size(); loopCounter++)
	{
		int from = rand() % output_vector.size();
		int to = rand() % output_vector.size();
		
		T temp = output_vector[from];
		output_vector[from] = output_vector[to];
		output_vector[to] = temp;
	}
	
	return output_vector;
}
