/*
 * MoGraSo: A programme for the calculation of the response of fibre grating
 * elements using modal methods.
 *
 * MoGraSo Copyright (C) 2007 Hannes Hochreiner <hannes@hochreiner.net>
 *
 * This file is part of MoGraSo.
 *
 * MoGraSo is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * MoGraSo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * MoGraSo.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "algorithms.h"
#include "layer.h"

int main()
{
	int ret = 0;
	
	layer temp(L"constant");

	std::map<std::wstring, std::complex<double> > vars = temp.get_vars();

	vars[L"electricIndex"] = 1.5;
	vars[L"magneticIndex"] = 1.0;
	vars[L"radius"] = 4.1e-6;
	
	temp.set_vars(vars);

	layerDescription<double> desc = temp.description<double>();
	
	ret = ret | !thresholdEqual(desc.n, 1.5);
	ret = ret | !thresholdEqual(desc.e, 1.5);
	ret = ret | !thresholdEqual(desc.u, 1.0);
	ret = ret | !thresholdEqual(desc.r, 4.1e-6);
	
	return ret;
}
