/*
 * MoGraSo: A programme for the calculation of the response of fibre grating
 * elements using modal methods.
 *
 * MoGraSo Copyright (C) 2007 Hannes Hochreiner <hannes@hochreiner.net>
 *
 * This file is part of MoGraSo.
 *
 * MoGraSo is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * MoGraSo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * MoGraSo.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include <complex>

#include "algorithms.h"

matrix<std::complex<double> > calculate_jacobian(const double& x, void* param);
matrix<std::complex<long double> > calculate_jacobian(const long double& x, void* param);

int main()
{
	int ret = 0;

	// Initialisation
	matrix<std::complex<double> > init(2,1);
	init(0) =  0.5;
	init(1) = -0.5;

	matrix<std::complex<double> > result = solve_ode_system(init, 0.0, 1.5, 1.0, &calculate_jacobian, 0, 1e-12);

	ret = ret | !thresholdEqual(result(0,0), 0.5 * std::exp(-3.0), 1e-3);
	
	matrix<std::complex<long double> > initL(2,1);
	initL(0) =  0.5L;
	initL(1) = -0.5L;

	matrix<std::complex<long double> > resultL = solve_ode_system(initL, 0.0L, 1.5L, 1.0L, &calculate_jacobian, 0, 1e-12L);

	ret = ret | !thresholdEqual(resultL(0,0), 0.5L * std::exp(-3.0L), 1e-3L);
	
	return ret;
}

matrix<std::complex<double> > calculate_jacobian(const double& x, void* param)
{
	matrix<std::complex<double> > out(2,2);

	out(0,0) = -1.0;
	out(0,1) =  1.0;
	out(1,0) =  1.0;
	out(1,1) = -1.0;

	return out;
}

matrix<std::complex<long double> > calculate_jacobian(const long double& x, void* param)
{
	matrix<std::complex<long double> > out(2,2);

	out(0,0) = -1.0L;
	out(0,1) =  1.0L;
	out(1,0) =  1.0L;
	out(1,1) = -1.0L;

	return out;
}

