/*
 * MoGraSo: A programme for the calculation of the response of fibre grating
 * elements using modal methods.
 *
 * MoGraSo Copyright (C) 2007 Hannes Hochreiner <hannes@hochreiner.net>
 *
 * This file is part of MoGraSo.
 *
 * MoGraSo is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * MoGraSo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * MoGraSo.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#ifndef _XMLFile
#define _XMLFile 1

#include <cstdlib>
#include <string>
#include <stdexcept>
#include <vector>
#include <map>

#include "stringFunctions.h"

#include <libxml/parser.h>
#include <libxml/tree.h>
#include <libxml/xpath.h>

inline void initiateXMLLibrary(void)
{
	LIBXML_TEST_VERSION
};

inline void cleanupXMLLibrary(void)
{
	xmlCleanupParser();
	xmlMemoryDump();
};

class XMLString
{
	private:
		xmlChar* f_xmlCharPtr;

	public:
		XMLString(const std::wstring& str);
		XMLString(xmlChar* xmlCharPtr);
		XMLString(const XMLString& xmlStr);
		operator std::wstring();
		operator xmlChar*();
		~XMLString();

	private:
		XMLString();
};

class XMLNode
{
	private:
		xmlNodePtr f_xmlNode;

	public:
		XMLNode();
		XMLNode(xmlNodePtr xmlNd);
		XMLNode(const XMLNode& xmlNd);
		~XMLNode();
		std::wstring get_property(const std::wstring& prop) const;
		std::wstring get_string() const;
};

class XPathObject
{
	private:
		xmlXPathObjectPtr f_xpathObj;

	public:
		XPathObject(xmlXPathObjectPtr objPtr);
		XPathObject(const XPathObject& obj);
		~XPathObject();
		int get_numberOfNodes() const;
		XMLNode get_node(const int& num) const;
		friend bool isNodeSet(const XPathObject& obj);

	private:
		XPathObject();
};

bool isNodeSet(const XPathObject& obj);

class XMLFile
{
	private:
		xmlXPathContextPtr f_XPathCtxt;
		xmlParserCtxtPtr f_parserCtxt;
		xmlDocPtr f_doc;
		void initiateParserContext(void) throw (std::runtime_error);
		void initiateXPathContext(void) throw (std::runtime_error);

	public:
		XMLFile(void);
		XMLFile(const std::wstring& filename);
		~XMLFile(void);
		void open(const std::wstring& filename);
		void close(bool throwExceptions_bool = true) throw (std::runtime_error);
		void saveAs(const std::wstring& filename);
//		XPathObject XPathQuery(xmlChar* query, xmlNs** nsList, int nsCount);
		XPathObject XPathQuery(const std::wstring& query, const std::wstring& prefix, const std::wstring& href);
		XPathObject XPathQuery(const std::wstring& query, const std::map<std::wstring, std::wstring>& namespaces);
		bool isOpen(void);
		const xmlDocPtr getXMLDocPtr(void);	
		
	private:
		XMLFile(const XMLFile&);
};

// std::wstring repConv(const xmlChar* xmlString);
// xmlChar* repConv(const std::wstring& wString);

#endif

