/*
 * MoGraSo: A programme for the calculation of the response of fibre grating
 * elements using modal methods.
 *
 * MoGraSo Copyright (C) 2007 Hannes Hochreiner <hannes@hochreiner.net>
 *
 * This file is part of MoGraSo.
 *
 * MoGraSo is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * MoGraSo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * MoGraSo.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#ifndef _MOGRASOFILEHANDLER_H
#define _MOGRASOFILEHANDLER_H 1

#include <iostream>
#include <sstream>
#include <string>
#include <vector>
#include <map>
#include <complex>

#include "XMLFile.h"
#include "stringFunctions.h"
#include "layer.h"
#include "section.h"
#include "variableStorage.h"
#include "outputStorage.h"
#include "layerStandIn.h"
#include "variableStandIn.h"

class mograsoFileHandler : public XMLFile
{
	private:

	public:
		mograsoFileHandler(void) : XMLFile() {};
		mograsoFileHandler(const std::wstring& filename) : XMLFile(filename) {};
		int get_numberOfJobs();
		std::wstring get_jobType(const int& job);
		std::vector<variableStandIn> get_variableStandInVector(const int& job);
		variableStorage get_variableStorage(const int& job);
		variableStorage variableStandInVectorToVariableStorage(const std::vector<variableStandIn>& varSIs);
		std::vector<layerStandIn> get_layerStandInVector(const int& job);
		std::vector<layer> get_layerVector(const int& job);
		static std::vector<layer> layerStandInVectorToLayerVector(const std::vector<layerStandIn>& layerSIs);
		std::vector<section> get_sectionVector(const int& job);
		std::map<std::wstring, std::wstring> get_options(const int& job);
		void get_perJobFiles(const int& job, const std::vector<std::wstring>& filenames, outputStorage* output);
		void get_perVarSetFiles(const int& job, const std::vector<std::wstring>& filenames, outputStorage* output);
		
	private:
		mograsoFileHandler(const mograsoFileHandler&);
};

#endif

