/*
 * MoGraSo: A programme for the calculation of the response of fibre grating
 * elements using modal methods.
 *
 * MoGraSo Copyright (C) 2007 Hannes Hochreiner <hannes@hochreiner.net>
 *
 * This file is part of MoGraSo.
 *
 * MoGraSo is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * MoGraSo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * MoGraSo.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "outputStorage.h"

outputStorage::outputStorage()
{
}

outputStorage::~outputStorage()
{
	for (std::map<std::wstring, pointerWrapper<std::ofstream> >::iterator it = f_perJob.begin(); it != f_perJob.end(); it++)
	{
		it->second.getObjectPtr()->close();
		delete it->second.getObjectPtr();
#ifdef HAVE_LIBPTHREAD
		pthread_mutex_destroy(it->second.getMutexPtr());
		delete it->second.getMutexPtr();
#endif
	}
}

void outputStorage::add_perJobFile(const std::wstring& dataId, const std::wstring& filename)
{
#ifdef HAVE_LIBPTHREAD
	pthread_mutex_t* mutex = new pthread_mutex_t;
	pthread_mutex_init(mutex, 0);
	f_perJob.insert(std::pair<std::wstring, pointerWrapper<std::ofstream> >(dataId, pointerWrapper<std::ofstream>(new std::ofstream(wstos(filename).c_str()), mutex)));
#else
	f_perJob.insert(std::pair<std::wstring, pointerWrapper<std::ofstream> >(dataId, pointerWrapper<std::ofstream>(new std::ofstream(wstos(filename).c_str()))));
#endif
}

void outputStorage::add_perVarSetFile(const std::wstring& dataId, const std::wstring& filename)
{
	f_perVarSet[dataId] = filename;
}

std::map<std::wstring, pointerWrapper<std::ofstream> > outputStorage::checkout_fileMap(const std::wstring& filenameSuffix) const
{
	std::map<std::wstring, pointerWrapper<std::ofstream> > out;
	out.insert(f_perJob.begin(), f_perJob.end());
	
	std::map<std::wstring, std::wstring> perVarSet = f_perVarSet;
	for (std::map<std::wstring, std::wstring>::iterator it = perVarSet.begin(); it != perVarSet.end(); it++)
#ifdef HAVE_LIBPTHREAD
		out.insert(std::pair<std::wstring, pointerWrapper<std::ofstream> >(it->first, pointerWrapper<std::ofstream>(new std::ofstream(wstos(incorporateFilenameSuffix(filenameSuffix, it->second)).c_str()), 0)));
#else
		out.insert(std::pair<std::wstring, pointerWrapper<std::ofstream> >(it->first, pointerWrapper<std::ofstream>(new std::ofstream(wstos(incorporateFilenameSuffix(filenameSuffix, it->second)).c_str()))));
#endif

	return out;
}

void outputStorage::checkin_fileMap(std::map<std::wstring, pointerWrapper<std::ofstream> >& map) const
{
	for (std::map<std::wstring, pointerWrapper<std::ofstream> >::iterator it = map.begin(); it != map.end(); it++)
	{
		if (f_perVarSet.count(it->first) > 0)
		{
			it->second.getObjectPtr()->close();
			delete it->second.getObjectPtr();
			map.erase(it);
		}
	}
}

std::wstring outputStorage::incorporateFilenameSuffix(const std::wstring& filenameSuffix, const std::wstring& filename) const
{
	std::wstring::size_type pos = filename.find(L".");
	
	if (pos == std::wstring::npos)
		return filename + filenameSuffix;
	else
		return filename.substr(0, pos) + filenameSuffix + filename.substr(pos, filename.size() - pos);
}
