/*
 * MoGraSo: A programme for the calculation of the response of fibre grating
 * elements using modal methods.
 *
 * MoGraSo Copyright (C) 2007 Hannes Hochreiner <hannes@hochreiner.net>
 *
 * This file is part of MoGraSo.
 *
 * MoGraSo is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * MoGraSo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * MoGraSo.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#ifndef _VARIABLESTORAGE_H
#define _VARIABLESTORAGE_H 1

#include <string>
#include <complex>
#include <map>
#include <stdexcept>
#include <vector>

#include "stringFunctions.h"

class variable
{
	private:
		std::wstring f_name;
		std::complex<double> f_start;
		std::complex<double> f_end;
		unsigned int f_numberOfSteps;
		std::wstring f_prefix;
		std::wstring f_suffix;
		int f_requiredPrecision;
	
	public:
		variable(const std::wstring& name, const std::complex<double>& start, const std::complex<double>& end = std::complex<double>(NAN, NAN), const unsigned long& numberOfSteps = 0, const std::wstring& prefix = L"", const std::wstring& suffix = L"");
		variable(const variable& obj);
		~variable();
		std::wstring get_name() const;
		unsigned long get_numberOfValues() const;
		std::complex<double> get_value(const unsigned long& number) const;
		std::wstring get_prefix() const;
		std::wstring get_suffix() const;
		int get_requiredPrecision() const;

	private:
		variable();
};

class variableStorage
{
	private:
		std::vector<variable> f_vars;
		std::map<unsigned long, unsigned long> f_order;
		unsigned long f_current;
	
	public:
		variableStorage();
		variableStorage(const variableStorage& obj);
		~variableStorage();
		void add_variable(const std::wstring& name, const std::complex<double>& start, const std::complex<double>& end = std::complex<double>(NAN, NAN), const unsigned long& numberOfSteps = 0, const std::wstring& prefix = L"", const std::wstring& suffix = L"");
		void add_variable(const variable& var);
		void init(const bool& randomise = false);
		unsigned long get_numberOfVarSets() const throw (std::runtime_error);
		std::map<std::wstring, std::complex<double> > get_varSet(const unsigned long& number) const throw (std::runtime_error);
		std::map<std::wstring, std::complex<double> > get_nextVarSet() throw (std::runtime_error);
		std::wstring get_filenameStringForVarSet(const unsigned long& number) const throw (std::runtime_error);
		std::wstring get_filenameStringForCurrentVarSet() const throw (std::runtime_error);
		std::wstring get_lineStartForVarSet(const unsigned long& number) const throw (std::runtime_error);
		int get_precisionForVar(const std::wstring& name) const;

	private:
		bool is_init() const;
};
#endif
