/*
 * MoGraSo: A programme for the calculation of the response of fibre grating
 * elements using modal methods.
 *
 * MoGraSo Copyright (C) 2007 Hannes Hochreiner <hannes@hochreiner.net>
 *
 * This file is part of MoGraSo.
 *
 * MoGraSo is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * MoGraSo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * MoGraSo.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "eo.h"

template <>
void eo<std::complex<double> >::set_vars(const std::map<std::wstring, std::complex<double> >& vars)
{
	if ((f_name != 0) && (vars.count(*f_name) != 0) && (!std::isnan(std::real(vars.find(*f_name)->second))) && (!std::isnan(std::imag(vars.find(*f_name)->second))))
	{
		delete f_value;
		f_value = new std::complex<double>(vars.find(*f_name)->second);
	}
	
	if (f_op1 != 0)
		f_op1->set_vars(vars);

	if (f_op2 != 0)
		f_op2->set_vars(vars);
}

template <>
void eo<std::complex<double> >::get_vars(std::map<std::wstring, std::complex<double> >& vars) const
{
	if (f_name != 0)
	{
		if (f_value != 0)
			vars.insert(std::pair<std::wstring, std::complex<double> >(*f_name, *f_value));
		else
			vars.insert(std::pair<std::wstring, std::complex<double> >(*f_name, std::complex<double>(NAN, NAN)));
	}
	
	if (f_op1 != 0)
		f_op1->get_vars(vars);
	
	if (f_op2 != 0)
		f_op2->get_vars(vars);
}

template <>
std::map<std::wstring, std::complex<double> > eo<std::complex<double> >::get_vars() const
{
	std::map<std::wstring, std::complex<double> > vars;
	
	if (f_name != 0)
	{
		if (f_value != 0)
			vars.insert(std::pair<std::wstring, std::complex<double> >(*f_name, *f_value));
		else
			vars.insert(std::pair<std::wstring, std::complex<double> >(*f_name, std::complex<double>(NAN, NAN)));
	}
	
	if (f_op1 != 0)
		f_op1->get_vars(vars);
	
	if (f_op2 != 0)
		f_op2->get_vars(vars);	
	
	return vars;
}
