/*
 * MoGraSo: A programme for the calculation of the response of fibre grating
 * elements using modal methods.
 *
 * MoGraSo Copyright (C) 2007 Hannes Hochreiner <hannes@hochreiner.net>
 *
 * This file is part of MoGraSo.
 *
 * MoGraSo is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * MoGraSo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * MoGraSo.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "stringFunctions.h"

std::string wstos(std::wstring str)
{
	return std::string(str.begin(), str.end());
}

std::wstring stows(std::string str)
{
	return std::wstring(str.begin(), str.end());
}

std::vector<std::wstring> tokenise(const std::wstring& str, const wchar_t& token)
{
	std::vector<std::wstring> vec;
	std::wstringstream stream(str);
	std::wstring temp;
	
	while (getline(stream, temp, token))
		vec.push_back(temp);
	
	return vec;
}

std::wstring complexDoubleToWString(const std::complex<double>& num, const int& precision)
{
	std::wstringstream strstream;
	
	if (precision > 0)
		strstream.precision(precision);
	
	if ((std::real(num) == 0.0) && (std::imag(num) == 0.0))
		strstream << L"0";
	else if ((std::real(num) != 0.0) && (std::imag(num) == 0.0))
		strstream << std::real(num);
	else if ((std::real(num) == 0.0) && (std::imag(num) != 0.0))
		strstream << std::imag(num) << L"i";
	else
	{
		strstream << std::real(num);
		if (std::imag(num) > 0.0)
			strstream << L"+";
		strstream << std::imag(num) << L"i";
	}
	
	return strstream.str();
}

std::complex<double> wstringToComplexDouble(const std::wstring& input)
{
	std::wstring::size_type i_sizeType = input.rfind(L"i");
	if (i_sizeType == std::wstring::npos)
		i_sizeType = input.rfind(L"I");
	
	std::wstring::size_type division_sizeType = 0;
	
	for (division_sizeType = input.find(L"+", division_sizeType + 1); (division_sizeType == 0) || (input[division_sizeType - 1] == L'e') || (input[division_sizeType - 1] == L'E'); division_sizeType = input.find(L"+", division_sizeType + 1)) {}
	
	if (division_sizeType == std::wstring::npos)
		for (division_sizeType = input.find(L"-", division_sizeType + 1); (division_sizeType == 0) || (input[division_sizeType - 1] == L'e') || (input[division_sizeType - 1] == L'E'); division_sizeType = input.find(L"-", division_sizeType + 1)) {}
	
	if (division_sizeType == std::wstring::npos)
	{
		// it's either pure real or pure imaginary
		double number_double;
		std::wstringstream temp(input);
		temp >> number_double;
		if (temp.fail())
			return std::complex<double>(NAN, NAN);

		if (i_sizeType == std::string::npos)
			return std::complex<double>(number_double, 0.0);
		else
			return std::complex<double>(0.0, number_double);
	}
	else
	{
		// it's complex
		double number1_double;
		std::wstringstream temp1(input.substr(0, division_sizeType));
		temp1 >> number1_double;
		if (temp1.fail())
			return std::complex<double>(NAN, NAN);

		double number2_double;
		std::wstringstream temp2(input.substr(division_sizeType, input.size() - division_sizeType));
		temp2 >> number2_double;
		if (temp2.fail())
			return std::complex<double>(NAN, NAN);
		
		if (i_sizeType > division_sizeType)
			return std::complex<double>(number1_double, number2_double);
		else
			return std::complex<double>(number2_double, number1_double);
	}
}

std::string seconds_to_string(double seconds)
{
	char buffer[33];
	unsigned int day = 60 * 60 * 24, hour = 60 * 60, minute = 60;
	if (seconds > day)
	{
		unsigned int days = (unsigned int)std::floor(seconds / day);
		sprintf(buffer, "%id ", days);
		return std::string(buffer) + seconds_to_string(seconds - days * day);
	}
	else if (seconds > hour)
	{
		unsigned int hours = (unsigned int)std::floor(seconds / hour);
		sprintf(buffer, "%ih ", hours);
		return std::string(buffer) + seconds_to_string(seconds - hours * hour);
	}
	else if (seconds > minute)
	{
		unsigned int minutes = (unsigned int)std::floor(seconds / minute);
		sprintf(buffer, "%im ", minutes);
		return std::string(buffer) + seconds_to_string(seconds - minutes * minute);
	}
	else
	{
		sprintf(buffer, "%.1fs", seconds);
		return std::string(buffer);
	}
}

std::wstring trim(const std::wstring& str)
{
	int first = 0;
	int last = str.size() - 1;

	for (; isspace(str[first]); first++) {}

	if (first == last)
		return std::wstring();
	
	for (; isspace(str[last]); last--) {}

	return str.substr(first, last - first + 1);
}

