/*
 * MoGraSo: A programme for the calculation of the response of fibre grating
 * elements using modal methods.
 *
 * MoGraSo Copyright (C) 2007 Hannes Hochreiner <hannes@hochreiner.net>
 *
 * This file is part of MoGraSo.
 *
 * MoGraSo is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * MoGraSo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * MoGraSo.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include <complex>
//#include <ctime>

#include "bessel_amos.h"

bool thresholdEqual(const std::complex<double>& num1, const std::complex<double>& num2, const double& threshold = 1e-15);
bool thresholdEqual(const double& num1, const double& num2, const double& threshold = 1e-15);

int main()
{
	int ret = 0;
	
	std::complex<double> C, Cp1;
	std::complex<double> arg(2.0, 0.05);
	double order = 0.0;

	besselj(order, arg, C, Cp1);
	
	ret = ret | !thresholdEqual(besselj(order, arg), C, 5e-14);
	ret = ret | !thresholdEqual(besselj(order + 1.0, arg), Cp1);

	bessely(order, arg, C, Cp1);
	ret = ret | !thresholdEqual(bessely(order, arg), C);
	ret = ret | !thresholdEqual(bessely(order + 1.0, arg), Cp1);

	besseli(order, arg, C, Cp1);
	ret = ret | !thresholdEqual(besseli(order, arg), C);
	ret = ret | !thresholdEqual(besseli(order + 1.0, arg), Cp1);

	besselk(order, arg, C, Cp1);
	ret = ret | !thresholdEqual(besselk(order, arg), C);
	ret = ret | !thresholdEqual(besselk(order + 1.0, arg), Cp1);

	besselh1(order, arg, C, Cp1);
	ret = ret | !thresholdEqual(besselh1(order, arg), C);
	ret = ret | !thresholdEqual(besselh1(order + 1.0, arg), Cp1);

	besselh2(order, arg, C, Cp1);
	ret = ret | !thresholdEqual(besselh2(order, arg), C, 5e-14);
	ret = ret | !thresholdEqual(besselh2(order + 1.0, arg), Cp1);

	arg = std::complex<double>(3.0, 0.02);
	order = 1.0;

	besselj(order, arg, C, Cp1);
	ret = ret | !thresholdEqual(besselj(order, arg), C);
	ret = ret | !thresholdEqual(besselj(order + 1.0, arg), Cp1);

	bessely(order, arg, C, Cp1);
	ret = ret | !thresholdEqual(bessely(order, arg), C);
	ret = ret | !thresholdEqual(bessely(order + 1.0, arg), Cp1);

	besseli(order, arg, C, Cp1);
	ret = ret | !thresholdEqual(besseli(order, arg), C, 1e-14);
	ret = ret | !thresholdEqual(besseli(order + 1.0, arg), Cp1);

	besselk(order, arg, C, Cp1);
	ret = ret | !thresholdEqual(besselk(order, arg), C);
	ret = ret | !thresholdEqual(besselk(order + 1.0, arg), Cp1);

	besselh1(order, arg, C, Cp1);
	ret = ret | !thresholdEqual(besselh1(order, arg), C);
	ret = ret | !thresholdEqual(besselh1(order + 1.0, arg), Cp1);

	besselh2(order, arg, C, Cp1);
	ret = ret | !thresholdEqual(besselh2(order, arg), C);
	ret = ret | !thresholdEqual(besselh2(order + 1.0, arg), Cp1);

// 	time_t start = time(0);
// 	start = time(0);
// 	for (int counter = 0; counter < 1000000; counter++)
// 	{
// 		besselj(order, arg);
// 		besselj(order + 1, arg);
// 	}
// 	std::cout << "J: " << difftime(time(0), start);
// 
// 	start = time(0);
// 	for (int counter = 0; counter < 1000000; counter++)
// 		besselj(order, arg, C, Cp1);
// 	std::cout << " " << difftime(time(0), start) << "\n";
// 
// 	start = time(0);
// 	for (int counter = 0; counter < 100000; counter++)
// 	{
// 		bessely(order, arg);
// 		bessely(order + 1, arg);
// 	}
// 	std::cout << "Y: " << difftime(time(0), start);
// 
// 	start = time(0);
// 	for (int counter = 0; counter < 100000; counter++)
// 		bessely(order, arg, C, Cp1);
// 	std::cout << " " << difftime(time(0), start) << "\n";
// 
// 	start = time(0);
// 	for (int counter = 0; counter < 1000000; counter++)
// 	{
// 		besseli(order, arg);
// 		besseli(order + 1, arg);
// 	}
// 	std::cout << "I: " << difftime(time(0), start);
// 
// 	start = time(0);
// 	for (int counter = 0; counter < 1000000; counter++)
// 		besseli(order, arg, C, Cp1);
// 	std::cout << " " << difftime(time(0), start) << "\n";
// 	
// 	start = time(0);
// 	for (int counter = 0; counter < 1000000; counter++)
// 	{
// 		besselk(order, arg);
// 		besselk(order + 1, arg);
// 	}
// 	std::cout << "K: " << difftime(time(0), start);
// 
// 	start = time(0);
// 	for (int counter = 0; counter < 1000000; counter++)
// 		besselk(order, arg, C, Cp1);
// 	std::cout << " " << difftime(time(0), start) << "\n";

	return ret;
}

bool thresholdEqual(const std::complex<double>& num1, const std::complex<double>& num2, const double& threshold)
{
	if (thresholdEqual(std::real(num1), std::real(num2), threshold) && thresholdEqual(std::real(num1), std::real(num2), threshold))
		return true;
	else
		return false;
}

bool thresholdEqual(const double& num1, const double& num2, const double& threshold)
{
	if (num1 == 0.0)
		if (std::abs(num2) < threshold)
			return true;
		else
			return false;

	if (num2 == 0.0)
		if (std::abs(num1) < threshold)
			return true;
		else
			return false;
	
	double ratio = std::abs(num1 / num2);
	if (ratio == 1.0)
		return true;
	else if (ratio < 1.0)
		ratio = 1.0 - ratio;
	else if (ratio > 1.0)
		ratio--;
	
	if (ratio < threshold)
		return true;
	else
		return false;
}
